@@ -49,6 +49,7 @@ struct _OobsUsersConfigPrivate
   uid_t     maximum_uid;
   gchar    *default_shell;
   gchar    *default_home;
+  guint     id;
 
   OobsGroup *default_group;
 };
@@ -152,6 +153,7 @@ oobs_users_config_init (OobsUsersConfig *config)
   priv = OOBS_USERS_CONFIG_GET_PRIVATE (config);
 
   priv->users_list = _oobs_list_new (OOBS_TYPE_USER);
+  config->_priv = priv;
 }
 
 static void
@@ -159,7 +161,7 @@ free_configuration (OobsUsersConfig *config)
 {
   OobsUsersConfigPrivate *priv;
 
-  priv = OOBS_USERS_CONFIG_GET_PRIVATE (config);
+  priv = config->_priv;
 
   oobs_list_clear (priv->users_list);
   g_free (priv->default_shell);
@@ -180,7 +182,7 @@ oobs_users_config_finalize (GObject *object)
 
   g_return_if_fail (OOBS_IS_USERS_CONFIG (object));
 
-  priv = OOBS_USERS_CONFIG_GET_PRIVATE (object);
+  priv = OOBS_USERS_CONFIG (object)->_priv;
 
   if (priv)
     {
@@ -204,7 +206,7 @@ oobs_users_config_set_property (GObject      *object,
 
   g_return_if_fail (OOBS_IS_USERS_CONFIG (object));
 
-  priv = OOBS_USERS_CONFIG_GET_PRIVATE (object);
+  priv = OOBS_USERS_CONFIG (object)->_priv;
 
   switch (prop_id)
     {
@@ -235,7 +237,7 @@ oobs_users_config_get_property (GObject      *object,
 
   g_return_if_fail (OOBS_IS_USERS_CONFIG (object));
 
-  priv = OOBS_USERS_CONFIG_GET_PRIVATE (object);
+  priv = OOBS_USERS_CONFIG (object)->_priv;
 
   switch (prop_id)
     {
@@ -261,16 +263,21 @@ static OobsUser*
 create_user_from_dbus_reply (OobsObject      *object,
 			     DBusMessage     *reply,
 			     DBusMessageIter  struct_iter,
-			     GHashTable      *hashtable)
+			     GHashTable      *hashtable,
+			     guint           *max_id)
 {
   OobsUser *user;
   DBusMessageIter iter, gecos_iter;
   int    uid, gid;
+  guint  id;
   gchar *login, *passwd, *home, *shell;
   gchar *name, *room_number, *work_phone, *home_phone, *other_data;
 
   dbus_message_iter_recurse (&struct_iter, &iter);
 
+  dbus_message_iter_get_basic (&iter, &id);
+  dbus_message_iter_next (&iter);
+
   dbus_message_iter_get_basic (&iter, &login);
   dbus_message_iter_next (&iter);
   
@@ -323,6 +330,10 @@ create_user_from_dbus_reply (OobsObject      *object,
 		       "other-data", other_data,
 		       NULL);
 
+  /* set the id by hand */
+  user->id = id;
+  *max_id = MAX (id, *max_id);
+
   /* put the GID in the hashtable, when the users list has
    * been completely generated, we may query the groups safely */
   g_hash_table_insert (hashtable,
@@ -362,6 +373,7 @@ create_dbus_struct_from_user (OobsUser        *user,
 
   dbus_message_iter_open_container (array_iter, DBUS_TYPE_STRUCT, NULL, &struct_iter);
 
+  dbus_message_iter_append_basic (&struct_iter, DBUS_TYPE_UINT32,  &user->id);
   utils_append_string (&struct_iter, login);
   utils_append_string (&struct_iter, password);
   dbus_message_iter_append_basic (&struct_iter, DBUS_TYPE_INT32,  &uid);
@@ -456,7 +468,7 @@ query_groups (OobsUsersConfig *users_config,
   if (default_gid > 0)
     {
       group = get_group_with_gid (OOBS_GROUPS_CONFIG (groups_config), default_gid);
-      OOBS_USERS_CONFIG_GET_PRIVATE (users_config)->default_group = group;
+      ((OobsUsersConfigPrivate *) users_config->_priv)->default_group = group;
 
       if (group)
 	g_object_unref (group);
@@ -476,10 +488,12 @@ oobs_users_config_update (OobsObject *object)
   gchar           *str;
   GHashTable      *hashtable;
   gint             default_gid;
+  guint            id;
 
-  priv  = OOBS_USERS_CONFIG_GET_PRIVATE (object);
+  priv  = OOBS_USERS_CONFIG (object)->_priv;
   reply = _oobs_object_get_dbus_message (object);
   hashtable = g_hash_table_new_full (NULL, NULL, g_object_unref, NULL);
+  id = 0;
 
   /* First of all, free the previous configuration */
   free_configuration (OOBS_USERS_CONFIG (object));
@@ -490,7 +504,7 @@ oobs_users_config_update (OobsObject *object)
   while (dbus_message_iter_get_arg_type (&elem_iter) == DBUS_TYPE_STRUCT)
     {
       user = G_OBJECT (create_user_from_dbus_reply (object, reply,
-						    elem_iter, hashtable));
+						    elem_iter, hashtable, &id));
 
       oobs_list_append (priv->users_list, &list_iter);
       oobs_list_set    (priv->users_list, &list_iter, G_OBJECT (user));
@@ -499,6 +513,8 @@ oobs_users_config_update (OobsObject *object)
       dbus_message_iter_next (&elem_iter);
     }
 
+  priv->id = id;
+
   dbus_message_iter_next (&iter);
   priv->shells = utils_get_string_list_from_dbus_reply (reply, iter);
 
@@ -538,13 +554,14 @@ oobs_users_config_commit (OobsObject *object)
   gboolean valid, correct;
   gint default_gid;
 
-  priv = OOBS_USERS_CONFIG_GET_PRIVATE (object);
+  priv = OOBS_USERS_CONFIG (object)->_priv;
   message = _oobs_object_get_dbus_message (object);
 
   dbus_message_iter_init_append (message, &iter);
   dbus_message_iter_open_container (&iter,
 				    DBUS_TYPE_ARRAY,
 				    DBUS_STRUCT_BEGIN_CHAR_AS_STRING
+				    DBUS_TYPE_UINT32_AS_STRING
 				    DBUS_TYPE_STRING_AS_STRING
 				    DBUS_TYPE_STRING_AS_STRING
 				    DBUS_TYPE_INT32_AS_STRING
@@ -619,7 +636,7 @@ oobs_users_config_get_users (OobsUsersConfig *config)
   g_return_val_if_fail (config != NULL, NULL);
   g_return_val_if_fail (OOBS_IS_USERS_CONFIG (config), NULL);
 
-  priv = OOBS_USERS_CONFIG_GET_PRIVATE (config);
+  priv = config->_priv;
 
   return priv->users_list;
 }
@@ -632,7 +649,7 @@ oobs_users_config_get_minimum_users_uid (OobsUsersConfig *config)
   g_return_val_if_fail (config != NULL, OOBS_MAX_UID);
   g_return_val_if_fail (OOBS_IS_USERS_CONFIG (config), OOBS_MAX_UID);
 
-  priv = OOBS_USERS_CONFIG_GET_PRIVATE (config);
+  priv = config->_priv;
 
   return priv->minimum_uid;
 }
@@ -654,7 +671,7 @@ oobs_users_config_get_maximum_users_uid (OobsUsersConfig *config)
   g_return_val_if_fail (config != NULL, OOBS_MAX_UID);
   g_return_val_if_fail (OOBS_IS_USERS_CONFIG (config), OOBS_MAX_UID);
 
-  priv = OOBS_USERS_CONFIG_GET_PRIVATE (config);
+  priv = config->_priv;
 
   return priv->maximum_uid;
 }
@@ -676,7 +693,7 @@ oobs_users_config_get_default_shell (OobsUsersConfig *config)
   g_return_val_if_fail (config != NULL, NULL);
   g_return_val_if_fail (OOBS_IS_USERS_CONFIG (config), NULL);
 
-  priv = OOBS_USERS_CONFIG_GET_PRIVATE (config);
+  priv = config->_priv;
 
   return priv->default_shell;
 }
@@ -698,7 +715,7 @@ oobs_users_config_get_default_home_dir (OobsUsersConfig *config)
   g_return_val_if_fail (config != NULL, NULL);
   g_return_val_if_fail (OOBS_IS_USERS_CONFIG (config), NULL);
 
-  priv = OOBS_USERS_CONFIG_GET_PRIVATE (config);
+  priv = config->_priv;
 
   return priv->default_home;
 }
@@ -720,7 +737,7 @@ oobs_users_config_get_default_group (OobsUsersConfig *config)
   g_return_val_if_fail (config != NULL, NULL);
   g_return_val_if_fail (OOBS_IS_USERS_CONFIG (config), NULL);
 
-  priv = OOBS_USERS_CONFIG_GET_PRIVATE (config);
+  priv = config->_priv;
 
   return priv->default_group;
 }
@@ -733,7 +750,18 @@ oobs_users_config_get_available_shells (OobsUsersConfig *config)
   g_return_val_if_fail (config != NULL, NULL);
   g_return_val_if_fail (OOBS_IS_USERS_CONFIG (config), NULL);
 
-  priv = OOBS_USERS_CONFIG_GET_PRIVATE (config);
+  priv = config->_priv;
 
   return priv->shells;
 }
+
+guint
+_oobs_users_config_get_id (OobsUsersConfig *config)
+{
+  OobsUsersConfigPrivate *priv;
+
+  priv = config->_priv;
+
+  /* FIXME: this could overflow */
+  return ++priv->id;
+}
